@@ -19,6 +19,9 @@
  *	IT MIGHT NOT WORK FOR YOU.  
  *	
  *	Version 0.1alpha: initial release
+ *	Version 0.2alpha: dropped the buggy scan command. Using
+ *			  the command for reading single lines instead
+ *			   
  *
  * -----------------------------------------------------------------------------------
  */
@@ -33,7 +36,7 @@
 #include <sys/types.h>
 #include <unistd.h>
 
-#include "/usr/src/linux/drivers/scsi/sg.h"
+#include "/usr/src/linux/include/scsi/sg.h"
 #include "coolscan.h"
 
 int fd;
@@ -74,7 +77,7 @@ int sg_xfer(u_char *src,int len,u_char *dest,
                 return -1;
         }
         result = ((struct sg_header*)buf)->result;
-        if (result) {
+        if (result && debug) {
                 fprintf(stderr,"sg_xfer: result is %d\n",result);
         }
         if (count > sizeof(struct sg_header)) {
@@ -89,7 +92,8 @@ int sg_xfer(u_char *src,int len,u_char *dest,
         return count;
 }
 
-int inquire_device(char *mfg,char *model)
+int 
+inquire_device(char *mfg,char *model)
 {
         static unsigned char inquiry[] = { 0x12,0,0,0,0x24,0 };
         unsigned char reply[256];
@@ -108,14 +112,16 @@ int inquire_device(char *mfg,char *model)
  *
  */
 
-int read_data_10(unsigned char type,unsigned char *dest,int len)
+int 
+read_data_10(unsigned char type,unsigned char *dest,int len)
 {
         u_char read_10[10] = { 0x28,0,type,0,0,0,0,len >> 8,len,0 };
         if ((len=sg_xfer(read_10,10,dest,len)) < 0)
                 return -1;
 	return len;
 }
-int write_data_10(char *data,int len)
+int 
+write_data_10(char *data,int len)
 {	
 	static u_char write_10[256] = { 0x2a,0,0,0,0,0,0,0,0,0}; 
 	write_10[7] = len >> 8; write_10[8] = len;
@@ -124,7 +130,8 @@ int write_data_10(char *data,int len)
                 return -1;
         return 0;
 }
-int test_unit_ready( ){
+int 
+test_unit_ready( ){
 	static u_char test_unit_ready[6] = { 0,0,0,0,0,0 };
 	struct sg_header dest;
 	int i,count;
@@ -137,18 +144,19 @@ int test_unit_ready( ){
 	printf("reply_len %d,result %d\n",dest.reply_len,dest.result);
 	return (int) dest.sense_buffer[0];
 }
-void write_string(unsigned char *format,...)
+void 
+write_string(unsigned char *format,...)
 {
         va_list args;
-        unsigned char buf[256];
+        static unsigned char buf[256];
         va_start(args, format);
         vsprintf(buf,format,args);
         write_data_10(buf,strlen(buf));
 }
-int is_coolscan(char *devname){
-	
+int 
+is_coolscan(char *devname)
+{
 	fd = open(devname, O_RDWR);
-
         if (fd >= 0 ) {	
 		int type;
                 char mfr[9], model[17];
@@ -158,12 +166,17 @@ int is_coolscan(char *devname){
 		if(!quiet){
                 	fprintf(stderr,"Model:%s\nVendor:%s\n",model,mfr);
 		}
-                if (type == 6 && !strncmp(mfr,"Nikon",5))
-                        return 1;
+                if (type == 6 && !strncmp(mfr,"Nikon",5)) {
+			if(strncmp(model,"COOLSCAN",8)) {
+				fprintf(stderr,"This is not a Coolscan LS10 !\n");
+			}
+			else {
+                        	return 1;
+			}
+		}
                 close(fd);
         }
         return -1;
-	
 }
 
 int find_coolscan(void)
@@ -179,6 +192,16 @@ int find_coolscan(void)
                         return 1;
                 }
         }
+	for (i='0'; i <= '9'; i++) {
+                sprintf(buf,"/dev/sg%c",i);
+                if ( is_coolscan(buf) != -1) {
+                        if (!quiet)
+                                fprintf(stderr,
+                                        "Coolscan found on '%s'\n",buf);
+                        return 1;
+                }
+        }
+
         return -1;
 }
 
@@ -188,12 +211,15 @@ int find_coolscan(void)
  *
  */
 
-void wait_for_ack(int timeout){
+void wait_for_ack(int timeout)
+{
 
 	unsigned char dest[256];
 	dest[0]=0;
-	while( (dest[0]!=6) && (timeout-- !=0))
+	do {
 		read_data_10(1,dest,10);
+		/*if(dest[0]!=6)*((char*)0)=0;*/
+	} while( (dest[0] != 6) && (timeout-- !=0) );
 	if(timeout == 0)fprintf(stderr,"timeout waiting for acknowledge\n");  
 	
 }
@@ -238,6 +264,7 @@ int get_data_block(int type,unsigned char *buf,int buflen){
         /* check line data */
         if ( buf[len-1] == 0x03) {
                 /* final message detected */
+		if(debug) fprintf(stderr,"endcode found \n");
                 return len-1;
         }
 
@@ -362,7 +389,7 @@ void register_color_balance(int k, int red, int green, int blue,int film_type ){
  */
 
 void specify_color_balance(int which){
-	write_string("US%d\015\013",which);
+	write_string("US%d\015\012",which);
 }
 void read_color_balance(int which){ char data[20];
 	int i;
@@ -397,57 +424,61 @@ void line_sequential_output(){
 void pixel_interleave_output(){
 	write_string("S_RGBD\015\012");
 }
-
 void mono_output_color(char c){
-	if(c == 'r' || c == 'b' || c == 'g'){
-		write_string("F%c\015\012",c);
+	if(c == 'R' || c == 'G' || c == 'B'){
+		write_string("F%c,R\015\012",c);
+		wait_for_ack(50);
 	}
+	
 }
 /* 
  *	Scans an area starting at x1,y1 
  *	with a pitch ot px,py
  *	nx*ny pixels will be scanned.
- *	The output format is ppm or pgm.
- *	the manual says that there will be a block of the len 1 and ETX (end text
- *	= 0x03 ) after the last block, but there is none.
- *	I also get one line fewer than expected, so I call this with 
- *	1 line more than required. The manual doesn't say that I have
- *	to wait for acknowledge after the BX command.
+ *      I now use the command for reading single lines because 
+ * 	the command for reading a full picture is totaly broken	
  */
 
-void scan( char *filename,int format,int x1,int y1,int px,int py,int nx,int ny){
-	static u_char data[8000];
-	int lines=nx,len;
-	FILE *file;
-	data[2]=0;
-	file=fopen(filename,"w");
-	if(format == FORMAT_PPM){
-		fprintf(file,"P6\n# Coolscan/Linux\n%d %d\n255\n",ny,nx);
+int 
+newscan(char *filename,int format,int x1,int y1,int px,int py,int nx,int ny) {
+	int line;
+	static u_char data[9000];
+        FILE *file;
+        data[2]=0;
+        file=fopen(filename,"w");
+	if(file==NULL) {
+		fprintf(stderr,"can't open outputfile %s\n",filename);
+		return -1;
 	}
-	if(format == FORMAT_PGM){
-		fprintf(file,"P5\n# Coolscan/Linux\n%d %d\n255\n",ny,nx);
+	else {
+		fprintf(stderr,"Writing To File: \"%s\"\n",filename);
 	}
-	write_string("BX %d,%d,%d,%d,%d,%d\015\012",x1,y1,px,py,nx+1,ny);
-	wait_for_ack(50); 
-	while( (lines-- > 0) && (( len = get_data_block(0,data,8000))>0)  ){
-		if(debug)
-			fprintf(stderr,"length %d, endcode 0x%x \n",len,data[len]);
-		fwrite(data,1,len,file);	
-
-		/* May be this helps speeding up the
-		   bus because there is no disconnect 
-		   possible on my machine, because of an 
-		   very old SCSI I Harddisk. 
-		*/
-		 usleep(50000);
+        if(format == FORMAT_PPM){
+                fprintf(file,"P6\n# Coolscan/Linux\n%d %d\n255\n",ny,nx);
+        }
+        if(format == FORMAT_PGM){
+                fprintf(file,"P5\n# Coolscan/Linux\n%d %d\n255\n",ny,nx);
+        }
+	for(line =x1;line<(x1+nx*px);line+=px) {
+		read_line(line,y1,py,ny,data);
+		if(format==FORMAT_PPM) {
+                	fwrite(data,1,ny*3,file);
+		}
+		else if (format == FORMAT_PGM) {
+                	fwrite(data,1,ny,file);
+		}
+		fprintf(stderr,"\015Current Line: %d ",line);
 	}
-	fclose(file);
-	
+	fprintf(stderr,"\n");
+        fclose(file);
+	stage_move_to(-1);
+	return 0;
 }
-void read_line(int line,char *buf){
+void 
+read_line(int line,int y1,int py,int ny,char *buf){
 	int len;
 	stage_move_to(line);
-        write_string("LN %d,%d,%d,%d\015\012",line,0,1,MAX_Y);
+        write_string("LN %d,%d,%d,%d\015\012",line,y1,py,ny);
         len = get_data_block(0,buf,9000);
 
         if(debug) fprintf(stderr,"The length %d, the endcode 0x%x \n",len,buf[len]);
@@ -457,7 +488,7 @@ void read_line(int line,char *buf){
 void print_line(int line){
 	static u_char buf[8000];
 	int i;
-	read_line( line,buf);
+	read_line( line,0,1,MAX_Y,buf);
 	for(i=0;i<MAX_Y;i+=10){
                 printf("%d ",buf[i]);
         }
@@ -699,6 +730,7 @@ void  init(struct cs_state_struct *s){
 	s->averaging = 0;
 	s->prescan_valid=0;
 	s->do_prescan = 0;
+	s->output_color = 'G';
 
 }
 void set_state( struct cs_state_struct *s){
@@ -727,7 +759,9 @@ void set_state( struct cs_state_struct *s){
 	else agof();
 	if(s->averaging)avon();
 	else avof(); 
-
+	if((s->film_type == MONO_POSITIVE)|( s->film_type == MONO_NEGATIVE)) {
+		mono_output_color(s->output_color);
+	}
 }
 #ifndef COOLSCAN_LIB
 
@@ -739,8 +773,8 @@ void print_help(){
 	"-mp                    monochrom positive\n"
 	"-mn                    monochrom negative\n"
 	"-pr			do prescan before scan\n"
-	"-time                  CCD-exposure time ( 25 - 100 )\n"
-	"-redtime               CCD-exposure time correction for ren   (25 - 100)\n"
+	"-time                  CCD-exposure time ( 25 - 100, default: 50 )\n"
+	"-redtime               CCD-exposure time correction for red   (25 - 100)\n"
 	"-greentime             CCD-exposure time correction for green (25 - 100)\n"
 	"-bluetime              CCD-exposure time correction for blue  (25 - 100)\n"
 	"-tv                    digital gamma correction for TV	\n"
@@ -754,8 +788,8 @@ void print_help(){
 	"-ej                    eject slide\n"
 	"-x1                    start coordinate for scan (0 - 3887) \n"
 	"-y1                    start coordinate for scan (0 - 2591) \n"
-	"-x2                    end coordinate for scan (x1+2*px - 3888) \n"
-	"-y2                    end coordinate for scan (y1+2*py - 2592) \n"
+	"-x2                    end coordinate for scan (x1+2*px - 3887) \n"
+	"-y2                    end coordinate for scan (y1+2*py - 2591) \n"
 	"-px                    pitch in x-direction ( 1 - 25 )\n"
 	"-py                    pitch in y-direction ( 1 - 25 )\n"
 	"-o                     outfile	\n"
@@ -901,8 +935,8 @@ int main(int argc,char * argv[]){
 	settings.ny = (settings.y2 - settings.y1 ) /settings.py;
 
 	read_error();
-        abort_scan();
-	read_empty();
+        abort_scan(); 
+	/*read_empty(); */
         pixel_interleave_output(); 
 	apin();
 
@@ -915,8 +949,9 @@ int main(int argc,char * argv[]){
 	{
 		struct cs_state_struct *s;
         	s = &settings;
-		scan(s->outfile,s->outputformat,s->x1,s->y1,s->px,s->py,s->nx,s->ny );
-		read_empty();
+		sleep(1);
+		newscan(s->outfile,s->outputformat,s->x1,s->y1,s->px,s->py,s->nx,s->ny );
+		/*read_empty();*/
 	}
 	exit(0);
 }
