@@ -37,9 +37,9 @@ main(){
 	linear_gamma();
 	for(k=0;k<50;k++){
 		unvary_parameter();
-		read_line(1000+40*k,uvbuf);
+		read_line(1000+40*k,0,1,MAX_Y,uvbuf);
 		vary_parameter();
-		read_line(1000+40*k,vbuf);
+		read_line(1000+40*k,0,1,MAX_Y,vbuf);
 		for(i=500;i<2000;i++){
 			j = uvbuf[i];
 			data[j].v_sum += vbuf[i];
