@@ -44,9 +44,9 @@ main(){
 
 	    for(k=0;k<1;k++){
 		unvary_parameter();
-		read_line(800+20*time+5*gtime ,uvbuf);
+		read_line(800+20*time+5*gtime ,0,1,MAX_Y,uvbuf);
 		vary_parameter();
-		read_line(800+20*time+5*gtime,vbuf);
+		read_line(800+20*time+5*gtime,0,1,MAX_Y,vbuf);
 		for(i=500;i<2000;i++){
 			j = uvbuf[i];
 			data[j].v_sum += vbuf[i];
